@@ -1,4 +1,4 @@
-/* Copyright 2017-2020 Fabian Koller
+/* Copyright 2017-2021 Fabian Koller
  *
  * This file is part of openPMD-api.
  *
@@ -25,6 +25,8 @@
 #include "openPMD/backend/Attribute.hpp"
 #include "openPMD/ChunkInfo.hpp"
 #include "openPMD/Dataset.hpp"
+#include "openPMD/IterationEncoding.hpp"
+#include "openPMD/Streaming.hpp"
 
 #include <memory>
 #include <map>
@@ -32,16 +34,14 @@
 #include <utility>
 #include <vector>
 
-#include "openPMD/Streaming.hpp"
-
 
 namespace openPMD
 {
-class Attributable;
+class AttributableInterface;
 class Writable;
 
 Writable*
-getWritable(Attributable*);
+getWritable(AttributableInterface*);
 
 /** Type of IO operation between logical and persistent data.
  */
@@ -65,6 +65,7 @@ OPENPMDAPI_EXPORT_ENUM_CLASS(Operation)
     WRITE_DATASET,
     READ_DATASET,
     LIST_DATASETS,
+    GET_BUFFER_VIEW,
 
     DELETE_ATT,
     WRITE_ATT,
@@ -73,7 +74,16 @@ OPENPMDAPI_EXPORT_ENUM_CLASS(Operation)
 
     ADVANCE,
     AVAILABLE_CHUNKS //!< Query chunks that can be loaded in a dataset
-}; // Operation
+}; // note: if you change the enum members here, please update docs/source/dev/design.rst
+
+namespace internal
+{
+    /*
+     * The returned strings are compile-time constants, so no worries about
+     * pointer validity.
+     */
+    std::string operationAsString( Operation );
+}
 
 struct OPENPMDAPI_EXPORT AbstractParameter
 {
@@ -107,7 +117,8 @@ template<>
 struct OPENPMDAPI_EXPORT Parameter< Operation::CREATE_FILE > : public AbstractParameter
 {
     Parameter() = default;
-    Parameter(Parameter const & p) : AbstractParameter(), name(p.name) {}
+    Parameter(Parameter const & p) :
+        AbstractParameter(), name(p.name), encoding(p.encoding) {}
 
     std::unique_ptr< AbstractParameter >
     clone() const override
@@ -117,13 +128,15 @@ struct OPENPMDAPI_EXPORT Parameter< Operation::CREATE_FILE > : public AbstractPa
     }
 
     std::string name = "";
+    IterationEncoding encoding = IterationEncoding::groupBased;
 };
 
 template<>
 struct OPENPMDAPI_EXPORT Parameter< Operation::OPEN_FILE > : public AbstractParameter
 {
     Parameter() = default;
-    Parameter(Parameter const & p) : AbstractParameter(), name(p.name) {}
+    Parameter(Parameter const & p) :
+        AbstractParameter(), name(p.name), encoding(p.encoding) {}
 
     std::unique_ptr< AbstractParameter >
     clone() const override
@@ -133,6 +146,12 @@ struct OPENPMDAPI_EXPORT Parameter< Operation::OPEN_FILE > : public AbstractPara
     }
 
     std::string name = "";
+    /*
+     * The backends might need to ensure availability of certain features
+     * for some iteration encodings, e.g. availability of ADIOS steps for
+     * variableBased encoding.
+     */
+    IterationEncoding encoding = IterationEncoding::groupBased;
 };
 
 template<>
@@ -281,8 +300,7 @@ template<>
 struct OPENPMDAPI_EXPORT Parameter< Operation::EXTEND_DATASET > : public AbstractParameter
 {
     Parameter() = default;
-    Parameter(Parameter const & p) : AbstractParameter(),
-        name(p.name), extent(p.extent) {}
+    Parameter(Parameter const & p) : AbstractParameter(), extent(p.extent) {}
 
     std::unique_ptr< AbstractParameter >
     clone() const override
@@ -291,7 +309,6 @@ struct OPENPMDAPI_EXPORT Parameter< Operation::EXTEND_DATASET > : public Abstrac
             new Parameter< Operation::EXTEND_DATASET >(*this));
     }
 
-    std::string name = "";
     Extent extent = {};
 };
 
@@ -408,6 +425,37 @@ struct OPENPMDAPI_EXPORT Parameter< Operation::LIST_DATASETS > : public Abstract
             = std::make_shared< std::vector< std::string > >();
 };
 
+template<>
+struct OPENPMDAPI_EXPORT Parameter< Operation::GET_BUFFER_VIEW > : public AbstractParameter
+{
+    Parameter() = default;
+    Parameter(Parameter const & p) : AbstractParameter(),
+        offset(p.offset), extent(p.extent), dtype(p.dtype), update(p.update),
+        out(p.out)
+    {}
+
+    std::unique_ptr< AbstractParameter >
+    clone() const override
+    {
+        return std::unique_ptr< AbstractParameter >(
+            new Parameter< Operation::GET_BUFFER_VIEW >(*this));
+    }
+
+    // in parameters
+    Offset offset;
+    Extent extent;
+    Datatype dtype = Datatype::UNDEFINED;
+    bool update = false;
+    // out parameters
+    struct OutParameters
+    {
+        bool backendManagedBuffer = false;
+        unsigned viewIndex = 0;
+        void *ptr = nullptr;
+    };
+    std::shared_ptr< OutParameters > out = std::make_shared< OutParameters >();
+};
+
 template<>
 struct OPENPMDAPI_EXPORT Parameter< Operation::DELETE_ATT > : public AbstractParameter
 {
@@ -565,7 +613,7 @@ public:
     { }
 
     template< Operation op >
-    explicit IOTask(Attributable* a,
+    explicit IOTask(AttributableInterface* a,
            Parameter< op > const & p)
             : writable{getWritable(a)},
               operation{op},
